/* mmw Copyright(C) Jason R. Fink jrf@rootshell.be SEE COPYING for license */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define MAXLEN 256
#define PROGRAM "mmw" 
#define VERSION "1.6"
#define MEMINFO "/proc/meminfo"

const unsigned int _parms_def = 5;

/* protos */
void version(void);
void usage(void);
void error_usage(void);
void helper(void);
void print_head(unsigned long int fws);
void read_meminfo(unsigned int polls, unsigned int interval, unsigned int sf);
void check_for_alphas(char *str);

/* simple reusable version print */
void version() { printf("%s %s\n", PROGRAM, VERSION); }
	
/* reusable usage print */
void usage()
{
	printf("usage: %s [polls] [interval]\n", PROGRAM);
	printf("usage: %s [option]\n", PROGRAM);
	printf("options are:\n");
	printf("  -h help\n");
	printf("  -v version\n");
}

/* simple reusable error usage print */
void error_usage()
{
	usage();
	exit(1);
}

/* simple reusable helper print */
void helper()
{
	version();
	printf("%s - Print out memory stats\n", PROGRAM);
	usage();
}

/* simple header */
void print_head(unsigned long int fws)
{
	int count;
	static char *header[]=
	{
		"total",
		"free",
		"shared",
		"buffer",
		"cached",
	};

	/* determine the field width for the header */	
	if(fws <= 100000) {
		for(count = 0; count <= 4; count++)
			printf("%-8s", header[count]);
	} else if(fws <= 100000000) {
		for(count = 0; count <= 4; count++)
			printf("%-11s", header[count]);
	} else if(fws > 100000000) {
		for(count = 0; count <= 4; count++)
			printf("%-14s", header[count]);
	} else {
		for(count = 0; count <= 4; count++)
			printf("%-15s", header[count]);
	}

	printf("\n");
}

/* recursively prints out mem information */
void read_meminfo(unsigned int polls, unsigned int interval, unsigned int sf)
{
	FILE *fp;
	static char ch[MAXLEN];
	unsigned long int mem_array[5];
	unsigned int count;

	if(polls != 0) {
		sleep(interval);

		/* typical file handling */
		if((fp = fopen(MEMINFO, "r")) == NULL) 
			fprintf(stderr, "could not open file %s\n", MEMINFO);

		while(fgets(ch, MAXLEN, fp) != NULL) {
			if(!strncmp(ch, "MemTotal:", 9)) {
				sscanf(ch + 10, "%lu", &mem_array[0]);
			} else if(!strncmp(ch, "MemFree:",8)) {
				sscanf(ch + 10, "%lu", &mem_array[1]);
			} else if(!strncmp(ch, "MemShared:", 10)) {
				sscanf(ch + 10, "%lu", &mem_array[2]);
			} else if(!strncmp(ch, "Buffers:", 8)) {
				sscanf(ch + 10, "%lu", &mem_array[3]);
			} else if(!strncmp(ch, "Cached:", 7)) {
				sscanf(ch + 10, "%lu", &mem_array[4]);
			}
		}

		fclose(fp);

		if(sf == 1) print_head(mem_array[0]);

		/* determine the field width for each printout */	
		if(mem_array[0] <= 100000) {
			for(count = 0; count <= 4; count++)
				printf("%-8li", mem_array[count]);
		} else if(mem_array[0] <= 100000000) {
			for(count = 0; count <= 4; count++)
				printf("%-11li", mem_array[count]);
		} else if(mem_array[0] > 100000000) {
			for(count = 0; count <= 4; count++)
				printf("%-14li", mem_array[count]);
		} else {
			for(count = 0; count <= 4; count++)
				printf("%-15li", mem_array[count]);
		}

		printf("\n");

		read_meminfo(polls - 1, interval, 0);

	}

}

/* make sure someone does not enter an alphanumeric by accident */
void check_for_alphas(char *str)
{
	unsigned int invalid = 0;

	if(strspn(str, "abcdefghijklmnopqrtsuvwxyz") >= 1) invalid++;
	if(strspn(str, "ABCDEFGHIJKLMNOPQRSTUVWXYZ") >= 1) invalid++;
	if(strspn(str, "~!@#$%^&*()&") >= 1) invalid++;
	if(strspn(str, ";:'\",.?") >=1) invalid++;

	if(invalid >= 1) {
		fprintf(stderr, "%s: invalid argument\n", str);
		error_usage();
	}
}
		

/* main */
int main(int argc, char *argv[])
{
	/* oops too much stuff */
	if(argc > 3) error_usage();

	/* default operation catch */
	if(argc == 1) {
		read_meminfo( _parms_def, _parms_def, 1);
		return 0;
	}

	/* very simple parser */
	if(!strncmp(argv[1], "-v", 2)) {
		version();
	} else if(!strncmp(argv[1], "-h", 2)) {
		helper();
	} else {
		if(argv[2] == NULL) {
			check_for_alphas(argv[1]);
			read_meminfo(atoi(argv[1]), _parms_def, 1);
		} else {
			check_for_alphas(argv[1]);
			check_for_alphas(argv[2]);
			read_meminfo(atoi(argv[1]), atoi(argv[2]), 1);
		}
	}

	return 0;
}


